﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;

public class BottomGameManager : Singleton<BottomGameManager>
{
    public enum PlayerColors
    {
        None = -1,
        Purple = 0,
        Green = 1,
        Yellow = 2,
        Pink = 3,
    }

    [Header("General")]
    public Text TimerText;
    public Text ProgressText;
    public Image TimerFillImage;
    public Button CloseButton;
    public Text PositivePointsText;
    public Text NegativePointsText;
    public float RotationSpeed = -1.0f;
    public Transform PointsTarget;

    [Header("Bottom Buttons")]
    public EnemyController[] EnemyControllers;
    public Sprite PinkSpritePressed;
    public Sprite PinkSpriteNormal;
    public Sprite YellowSpritePressed;
    public Sprite YellowSpriteNormal;
    public Sprite GreenSpritePressed;
    public Sprite GreenSpriteNormal;
    public Sprite PurpleSpritePressed;
    public Sprite PurpleSpriteNormal;

    public Color GreenColor;
    public Color PurpleColor;
    public Color YellowColor;
    public Color PinkColor;

    public Image SnakeMarker;
    public GameObject ParentGo;

    public AnimationCurve GameDurationCurve;
    private Vector3 _positiveStartsPos;
    public Vector3 LastBottomButtonClickedPosition;
    public int MyScore { get { return _myScore; } }

    private float _gameTimeRemaining;
    private bool _animatePositivePoints = false;
    private bool _animateNegativePoints = false;
    private float _elapsedTimeAnimatingPositivePoints;
    private float _elapsedTimeAnimatingNegativePoints;
    private float TIME_TO_ANIMATE_FLOATING_POINTS = 1.2f;
    private int _myScore = 0;
    private PlayerColors _targetColor;
    private Coroutine _snakeCoroutine;
    private Dictionary<PlayerColors, int> _playerIdColors = new Dictionary<PlayerColors, int>();

    private float TOTAL_TIME_FOR_ROUND { get { return GameDurationCurve.keys[GameDurationCurve.keys.Length - 1].time; } }

    private List<PlayerColors> _possibleColorsForNextTarget = new List<PlayerColors>();
    private bool _isGameRunning;
    public bool IsGameRunning { get { return _isGameRunning; } }

    public float CurrentTimeToWait { get { return GameDurationCurve.Evaluate(_gameTimeRemaining); } }

    private void OnEnable()
    {
        CloseButton.onClick.AddListener(OnCloseButtonClicked);
    }

    private void Update()
    {
        if (_isGameRunning)
        {
            _gameTimeRemaining -= Time.deltaTime;

            if (_gameTimeRemaining <= 0)
            {
                EndGame();
            }

            TimerText.text = string.Format("{0:0.0}s", _gameTimeRemaining);
            float delta = _gameTimeRemaining / TOTAL_TIME_FOR_ROUND;
            TimerFillImage.fillAmount = delta;
            if (delta < 0.2f)
            {
                TimerFillImage.color = new Color(1, 1, 1, 0.25f + Mathf.PingPong(Time.time * (delta * 10), 1));
            }
        }

        if (_animatePositivePoints)
        {
            _elapsedTimeAnimatingPositivePoints += Time.deltaTime;
            float delta = _elapsedTimeAnimatingPositivePoints / TIME_TO_ANIMATE_FLOATING_POINTS;
            PositivePointsText.transform.position = Vector3.Lerp(_positiveStartsPos, PointsTarget.position, delta);
            if (delta >= 1)
            {
                _animatePositivePoints = false;
                PositivePointsText.text = string.Empty;
            }
        }

        if (_animateNegativePoints)
        {
            _elapsedTimeAnimatingNegativePoints += Time.deltaTime;
            float delta = _elapsedTimeAnimatingNegativePoints / TIME_TO_ANIMATE_FLOATING_POINTS;
            NegativePointsText.transform.position = Vector3.Lerp(LastBottomButtonClickedPosition, PointsTarget.position, delta);
            if (delta >= 1)
            {
                _animateNegativePoints = false;
                NegativePointsText.text = string.Empty;
            }
        }
    }

    private void OnDisable()
    {
        CloseButton.onClick.RemoveListener(OnCloseButtonClicked);
    }

    private void OnCloseButtonClicked()
    {
        PUNManager.Instance.LeaveAndGotoWelcome();
        AudioManager.Instance.PlayAudioClip(AudioManager.Clip.SelectionClip);
    }

    public void StartSnake()
    {
        if (_snakeCoroutine != null)
        {
            StopCoroutine(_snakeCoroutine);
        }

        _snakeCoroutine = StartCoroutine(HandleSnake());
    }

    private IEnumerator HandleSnake()
    {
        while (!_isGameRunning)
        {
            yield return new WaitForEndOfFrame();
        }

        SetTargetColor();

        var color = SnakeMarker.color;
        var delta = 0f;
        var timer = 0f;
        while (delta != 1f)
        {
            timer += Time.deltaTime;
            delta = Mathf.Clamp01(timer / CurrentTimeToWait);
            color.a = delta;
            SnakeMarker.color = color;
            float scale = Mathf.Lerp(0.45f, 1.0f, delta);
            SnakeMarker.transform.localScale = new Vector3(scale, scale, scale);
            SnakeMarker.transform.Rotate(new Vector3(0, 0, RotationSpeed));
            yield return new WaitForEndOfFrame();
        }

        LoseProgress();
        LastBottomButtonClickedPosition = SnakeMarker.transform.position;
        SetupForNextPlayer();

        _snakeCoroutine = null;
    }

    private void SetTargetColor()
    {
        if (_possibleColorsForNextTarget.Count == 0)
        {
            SetAllEnemyColorsList();
        }

        if (_possibleColorsForNextTarget.Count == 1)
        {
            _targetColor = _possibleColorsForNextTarget[0];
        }
        else
        {
            _targetColor = _possibleColorsForNextTarget[UnityEngine.Random.Range(0, _possibleColorsForNextTarget.Count)];
        }

        _possibleColorsForNextTarget.Remove(_targetColor);
        SnakeMarker.color = GetColor(_targetColor);
    }

    public Color GetColor(PlayerColors color)
    {
        switch (color)
        {
            case PlayerColors.Green:
                return GreenColor;
            case PlayerColors.Purple:
                return PurpleColor;
            case PlayerColors.Yellow:
                return YellowColor;
            default:
                return PinkColor;
        }
    }

    private void LoseProgress()
    {
        int PointsToLose = 15;
        _myScore -= PointsToLose;

        if (_myScore < 0)
        {
            _myScore = 0;
        }

        UpdateProgressText();
        Handheld.Vibrate();
        _animateNegativePoints = true;
        _elapsedTimeAnimatingNegativePoints = 0;
        NegativePointsText.text = string.Format("-{0}", PointsToLose);
        AudioManager.Instance.PlayAudioClip(AudioManager.Clip.BadHitClip);
    }

    public void IncreaseProgress(int points)
    {
        _myScore += points;
        UpdateProgressText();
        _animatePositivePoints = true;
        _elapsedTimeAnimatingPositivePoints = 0;
        _positiveStartsPos = Input.mousePosition;
        PositivePointsText.text = string.Format("+{0}", points);
    }

    private void UpdateProgressText()
    {
        ProgressText.text = _myScore.ToString();
        PUNManager.Instance.UpdateScore();
    }

    public void UpdatePlayerScore(int playerId, int score)
    {
        if (playerId == PhotonNetwork.player.ID)
        {
            ProgressText.text = score.ToString();
            return;
        }

        foreach (var enemyController in EnemyControllers)
        {
            if (enemyController.UpdateText(playerId, score))
            {
                return;
            }
        }
    }

    public void EndGame()
    {
        _isGameRunning = false;
        TopGameManager.Instance.StopGame();
        _gameTimeRemaining = 0;
        UIManager.Instance.ShowScreen(UIManager.UIScreen.EndOfGame);
    }

    public void OnButtonClicked(PlayerColors playerColor, int playerID)
    {
        if (_targetColor != playerColor)
        {
            LoseProgress();
        }

        if (_targetColor != PlayerColors.None)
        {
            StopCoroutine(_snakeCoroutine);
            SetupForNextPlayer();
        }
    }

    private void SetupForNextPlayer()
    {
        SnakeMarker.color = Color.clear;

        PUNManager.Instance.SetNextPlayer(_playerIdColors[_targetColor]);
        _targetColor = PlayerColors.None;
        AudioManager.Instance.PlayAudioClip(AudioManager.Clip.GoodHitClip);
    }

    public void SetupGame()
    {
        _isGameRunning = false;

        _playerIdColors.Clear();
        _gameTimeRemaining = TOTAL_TIME_FOR_ROUND;
        _myScore = 0;

        var index = 0;
        var currentNetworkList = PhotonNetwork.playerList.ToList();
        currentNetworkList.Sort();

        for (int i = 0; i < currentNetworkList.Count; i++)
        {
            if (currentNetworkList[i] != PhotonNetwork.player)
            {
                _playerIdColors.Add((PlayerColors)i, currentNetworkList[i].ID);
                EnemyControllers[index].Setup((PlayerColors)i, currentNetworkList[i].ID);
                index++;
            }
        }

        SetAllEnemyColorsList();
    }

    public void StartGame()
    {
        _isGameRunning = true;
        SnakeMarker.color = Color.clear;
        SnakeMarker.transform.localScale = Vector3.zero;
    }

    private void SetAllEnemyColorsList()
    {
        _possibleColorsForNextTarget.Clear();

        var currentNetworkList = PhotonNetwork.playerList.ToList();
        currentNetworkList.Sort();
        for (int i = 0; i < currentNetworkList.Count; i++)
        {
            if (currentNetworkList[i] != PhotonNetwork.player)
            {
                _possibleColorsForNextTarget.Add((PlayerColors)i);
            }
        }
    }

    public Sprite GetNormalSprite(PlayerColors playerColors)
    {
        switch (playerColors)
        {
            case PlayerColors.Green:
                return GreenSpriteNormal;
            case PlayerColors.Pink:
                return PinkSpriteNormal;
            case PlayerColors.Purple:
                return PurpleSpriteNormal;
            case PlayerColors.Yellow:
                return YellowSpriteNormal;
            default:
                Debug.LogErrorFormat("No sprite set for {0}", playerColors);
                return GreenSpriteNormal;
        }
    }

    public Sprite GetPressedSprite(PlayerColors playerColor)
    {
        switch (playerColor)
        {
            case PlayerColors.Green:
                return GreenSpritePressed;
            case PlayerColors.Pink:
                return PinkSpritePressed;
            case PlayerColors.Purple:
                return PurpleSpritePressed;
            case PlayerColors.Yellow:
                return YellowSpritePressed;
            default:
                Debug.LogErrorFormat("No sprite set for {0}", playerColor);
                return GreenSpritePressed;
        }
    }
}