﻿using Photon;
using System.Collections;
using UnityEngine;
using Hashtable = ExitGames.Client.Photon.Hashtable;

public class PUNManager : PunBehaviour
{
    public const int MAX_PLAYERS = 4;
    public static PUNManager Instance { get { return _instance; } }
    public static PUNManager _instance;
    private Coroutine _setNextQuestionCoroutine;
    private const string SCORE_KEY = "Score";
    private const string NEXT_PLAYER_KEY = "PlayerNext";

    private const string GAME_VERSION = "1.0";

    private const string START_GAME_RPC = "StartGame";
    public const string NEXT_QUESTION_INDEX = "NextQuestionIndex";

    protected virtual void Awake()
    {
        _instance = gameObject.GetComponent<PUNManager>();
        PhotonNetwork.autoJoinLobby = true;
        PhotonNetwork.BackgroundTimeout = 0.5f;
    }

    public void Start()
    {
        Connect();
    }

    #region Game Setup/Cleanup
    [PunRPC]
    public void StartGame()
    {
        BottomGameManager.Instance.SetupGame();
        UIManager.Instance.WaitingForPlayersUIInstance.StartCountDownToStartGame();
    }

    public void LeaveAndGotoWelcome()
    {
        PhotonNetwork.LeaveRoom();
        if (BottomGameManager.Instance.IsGameRunning)
        {
            BottomGameManager.Instance.EndGame();
        }
        UIManager.Instance.ShowScreen(UIManager.UIScreen.Welcome);
    }

    public override void OnPhotonPlayerConnected(PhotonPlayer newPlayer)
    {
        if (PhotonNetwork.playerList.Length == MAX_PLAYERS)
        {
            SetNextPlayer();
            PhotonNetwork.room.IsOpen = false;
            photonView.RPC(START_GAME_RPC, PhotonTargets.All);
        }
    }

    public override void OnPhotonPlayerDisconnected(PhotonPlayer otherPlayer)
    {
        if (BottomGameManager.Instance.IsGameRunning)
        {
            BottomGameManager.Instance.UpdatePlayerScore(otherPlayer.ID, 0);
            BottomGameManager.Instance.EndGame();
        }
    }
    #endregion

    #region Data Transfer
    public override void OnPhotonCustomRoomPropertiesChanged(Hashtable propertiesThatChanged)
    {
        if (propertiesThatChanged.ContainsKey(NEXT_PLAYER_KEY))
        {
            if ((int)propertiesThatChanged[NEXT_PLAYER_KEY] == PhotonNetwork.player.ID)
            {
                BottomGameManager.Instance.StartSnake();
            }
        }

        if (propertiesThatChanged.ContainsKey(NEXT_QUESTION_INDEX))
        {
            var questionData = QuestionManager.Instance.GetQuestion((int)propertiesThatChanged[NEXT_QUESTION_INDEX]);
            TopGameManager.Instance.SetNextQuestion(questionData);
        }
    }

    public override void OnPhotonPlayerPropertiesChanged(object[] playerAndUpdatedProps)
    {
        for (int i = 0; i < playerAndUpdatedProps.Length; i += 2)
        {
            PhotonPlayer player = playerAndUpdatedProps[0] as PhotonPlayer;
            Hashtable props = playerAndUpdatedProps[1] as Hashtable;

            if (player == PhotonNetwork.player)
            {
                continue;
            }

            if (props.ContainsKey(SCORE_KEY))
            {
                BottomGameManager.Instance.UpdatePlayerScore(player.ID, (int)props[SCORE_KEY]);
            }
        }
    }

    #region Score Section
    public void UpdateScore()
    {
        var hashTable = new Hashtable
        {
            { SCORE_KEY, BottomGameManager.Instance.MyScore }
        };

        PhotonNetwork.player.SetCustomProperties(hashTable);
    }
    #endregion

    #region Next Question
    public void SetNextQuestion(float delay)
    {
        if (PhotonNetwork.isMasterClient)
        {
            _setNextQuestionCoroutine = StartCoroutine(DelayedSetNextQuestion(delay));
        }
    }

    private IEnumerator DelayedSetNextQuestion(float delay)
    {
        yield return new WaitForSeconds(delay);

        var hashTable = new Hashtable
        {
            { NEXT_QUESTION_INDEX, QuestionManager.Instance.GetRandomQuestionIndex() }
        };

        PhotonNetwork.room.SetCustomProperties(hashTable);

        _setNextQuestionCoroutine = null;
    }
    #endregion

    #region Set Player
    public void SetNextPlayer()
    {
        var hashTable = new Hashtable
        {
            { NEXT_PLAYER_KEY, PhotonNetwork.masterClient.ID }
        };

        PhotonNetwork.room.SetCustomProperties(hashTable);
    }

    public void SetNextPlayer(int nextPlayerId)
    {
        if (BottomGameManager.Instance.IsGameRunning)
        {
            var hashTable = new Hashtable
            {
                { NEXT_PLAYER_KEY, nextPlayerId }
            };

            PhotonNetwork.room.SetCustomProperties(hashTable);
        }
    }
    #endregion
    #endregion

    #region Networking
    public void Connect()
    {
        PhotonNetwork.ConnectUsingSettings(GAME_VERSION);
        UIManager.Instance.WelcomeUIInstance.StatusText.text = "Connecting to server...";
    }

    public void OnApplicationPause(bool pause)
    {
        if (!pause)
        {
            LeaveAndGotoWelcome();
            UIManager.Instance.WelcomeUIInstance.StartGameButton.gameObject.SetActive(false);
            PhotonNetwork.Disconnect();
        }
    }

    public override void OnConnectedToPhoton()
    {
        UIManager.Instance.WelcomeUIInstance.StatusText.text = "Connected to server.";
        UIManager.Instance.WelcomeUIInstance.StartGameButton.gameObject.SetActive(true);
    }

    public override void OnCreatedRoom()
    {
        UIManager.Instance.ShowScreen(UIManager.UIScreen.Waiting);
        QuestionManager.Instance.Init();
        SetNextQuestion(0);
    }

    public override void OnJoinedRoom()
    {
        UpdateScore();
        QuestionManager.Instance.Init();
        UIManager.Instance.ShowScreen(UIManager.UIScreen.Waiting);
    }

    public override void OnDisconnectedFromPhoton()
    {
        Connect();
    }
    #endregion
}